/*
   _===_ _           _                  ______ _     _             _ _ _
  |  ___| |         | |                 | ___ (_)   | |           (_) | |
  | |__ | | ___  ___| |_ _ __ ___ ______| |_/ /_  __| | ___  _   _ _| | | ___ _   _ _ __
  |  __|| |/ _ \/ __| __| '__/ _ \______| ___ \ |/ _` |/ _ \| | | | | | |/ _ \ | | | '__|
  | |___| |  __/ (__| |_| | | (_) |     | |_/ / | (_| | (_) | |_| | | | |  __/ |_| | |
  \____/|_|\___|\___|\__|_|  \___/      \____/|_|\__,_|\___/ \__,_|_|_|_|\___|\__,_|_|

  Bidouilleur PIC12F683 - exemple de contrôle d'un moteur pas-à-pas 
  Ajustement d'un condensateur de boucle magnétique.
  Électro-Bidouilleur, Novembre 2017


	TARGET device: PIC12F683
	Compilateur: SourceBoost's BoostC pour PIC
	IDE: SourceBoost IDE
*/
// INSTRUCTIONS AU COMPILATEUR
#include <PIC12F683.h>  // Probably redundant
#include <system.h>
#include <string.h>

// GLOBALS
unsigned char		speed_delay;
unsigned char		coil_step;
bool	 		    motor_on;
bool			    rot_sense;
		


// CONFIGURATION BYTES
#pragma DATA _CONFIG, _FCMEN_OFF & _IESO_OFF & _BOD_OFF & _CPD_OFF & _CP_OFF & _MCLRE_OFF & _PWRTE_OFF & _WDT_OFF & _INTOSCIO  
#pragma CLOCK_FREQ 8000000		// Tells the compiler what clock frequency is used. Important for delay functions

// CONSTANTS AND MASKS DEFINITION

	//Other definitions
#define coil_step1 				0b00010001	// The bit sequence to energize the coils on gpio
#define coil_step2 				0b00000001	// Half-step stepper sequence
#define coil_step3 				0b00100001	// "
#define coil_step4 				0b00100000	// "
#define coil_step5 				0b00100100	// "
#define coil_step6 				0b00000100	// "
#define coil_step7 				0b00010100	// "
#define coil_step8 				0b00010000	// "

#define Off_FastCcw_Thres		2			// Correspond to a comparator threshold of 0.42V
#define FastCcw_SlowCcw_Thres	6			// Correspond to a comparator threshold of 1.25V
#define SlowCcw_SlowCw_Thres	10			// Correspond to a comparator threshold of 2.08V
#define SlowCw_FastCw_Thres		15			// Correspond to a comparator threshold of 3.13V
#define fast_speed				1			// Corresponds to a delay between coil pulses in milliseconds for fast speeed
#define slow_speed				50			// Corresponds to a delay between coil pulses in milliseconds for slow speed						
#define ccw						false
#define cw						true
#define vr_mask					0b00001111	// positions in vrcon that set the comparator threshold voltage
#define compar_out				6			// bit position in cmcon0 that corresponds to the Cout 
#define coil_mask				0b00110101	// bit positions on gpio of the coils

// Interrupt Service Routine

//void interrupt( void )		// High priority interrupt
//{
//}										// End of interrupt routine
// 
// void interrupt_low( void )		// Low priority interrupt
//{
//}								// End of interrupt routine

void write_coils(char coil_step)
{
	gpio &= coil_step;  	//bit-wide AND function of coil_port and coil_step, result back into coil_port, This keeps already set bit and clears changing bit
	gpio |= coil_step;	//bit-wide OR function of coil_port and coil_step, result back into coil_port. This adds changing bit.
}

// EXECUTED CODE STARTS HERE


void main(void)
{
	// Initialization stage
	intcon 		= 	0b00000000; 		// disable interrupts.	
	ioc    		=	0b00000000;		// disable gpio interrupts on change
	pie1   		=	0b00000000; 		// disable various interrupts
	option_reg 	= 	0b00001111;		// weak pullups global disable, prescaler assigned to WDT and set to 1:128
	wpu    		=	0b00000000;		// individual weak pullups disabled
	osccon		=	0b11100001; 		// select internal oscillator and set it to 8Mhz	
	t1con		=	0b00110000;		// disable timer1 and set prescaler div. by 8 
	t2con		=	0b01111010;		// disable timer2 and set prescaler div. by 16 and postscaler div. by 16 
	adcon0		=	0b01000000;		// disable A/D converter
	ansel		=	0b00000010;		// Enable analog assignment to pin CIN- (GP1)
	cmcon0		=	0b00000100;		// enable comparator, select CIN- as inverting input, use internal reference, invert output.
	cmcon1 		=	0b00000000;		// comparator gating timer 1 asynchronously
	vrcon 		=	0b10100010;		// Enable internal comparator Vref, use low range, set Vref to 0.42V 
	wdtcon		=	0b00010100;		// Assign the right prescaler value to the watchdog timer. Keep WTD off for now.
	gpio 		=	0b00000000;		// Disable PTT (low), lows on other Port pins.
	trisio 		=	0b00001010;		// GPIO first disabled, but assigned as follows:    
									  //	5-Output to Stepper Motor Coil D
									  //	4-Output to Stepper Motor Coil C
									  //	3-Input only. Available
									  //	2-Output to Stepper Motor Coil B 
									  //	1-Comparator input-. Rotary Switch Input
									  //	0-Output to Stepper Motor Coil A
	delay_ms(250);					// Generates a startup delay of 0.25 second
	coil_step = 1; 
	while (true)	// Main loop 
	{
			// Sensing the rotary switch position
		vrcon &= ~(vr_mask);			// Set Vref to detect off on rotary switch 
		vrcon |= Off_FastCcw_Thres;		// " 
		delay_ms(1);				// required for comparator to stabilize
		if (test_bit(cmcon0,compar_out))
		{
			motor_on = false;
		}
		else
		{
			motor_on = true;
			vrcon &= ~(vr_mask);				// Set Vref to detect Fast CCW on rotary switch 
			vrcon |=  FastCcw_SlowCcw_Thres;		// " 
     		delay_ms(1);				// required for comparator to stabilize
			if (test_bit(cmcon0,compar_out))
			{
				speed_delay = fast_speed;		// Switch is on Fast Speed CCW rotation
				rot_sense = ccw;
			}
			else 
			{
				vrcon &= ~(vr_mask);			// Set Vref to detect Slow CCW on rotary switch 
				vrcon |=  SlowCcw_SlowCw_Thres;		// " 
				delay_ms(1);				// required for comparator to stabilize
				if (test_bit(cmcon0,compar_out))
				{
					speed_delay = slow_speed;	// Switch is on Slow Speed CCW rotation
					rot_sense = ccw;
				}
				else 
				{
					vrcon &= ~(vr_mask);	// Set Vref to detect Slow CW on rotary switch 
					vrcon |=  SlowCw_FastCw_Thres;		// " 
					delay_ms(1);				// required for comparator to stabilize
					if (test_bit(cmcon0,compar_out))
					{
						speed_delay = slow_speed;	// Switch is on Slow Speed CW rotation
						rot_sense = cw;
					}
					else    // Switch is on Fast Speed CW rotation
					{
						speed_delay = fast_speed;
						rot_sense = cw;
					}
				}
			}
		}
			// Energize the coils for one motor rotation
		if (!motor_on)				// Case when motor is off
		{
			gpio &= ~coil_mask;		// Clears the coil outputs
		}		
		else if (motor_on && rot_sense)   	// Case when rotation is clockwise Slower speeds, Half step
		{
			if (coil_step == 1)
			{
				write_coils(coil_step1);
				coil_step = 2;
			}
			else if (coil_step == 2)
			{
				write_coils(coil_step2);
				coil_step = 3;
			}
			else if (coil_step == 3)
			{
				write_coils(coil_step3);
				coil_step = 4;
			}
			else if (coil_step == 4)
			{
				write_coils(coil_step4);
				coil_step = 5;
			}
			else if (coil_step == 5)
			{
				write_coils(coil_step5);
				coil_step = 6;
			}
			else if (coil_step == 6)
			{
				write_coils(coil_step6);
				coil_step = 7;
			}
			else if (coil_step == 7)
			{
				write_coils(coil_step7);
				coil_step = 8;
			}
			else
			{
				write_coils(coil_step8);
				coil_step = 1;
			}
			delay_ms(speed_delay);			// generates a delay
		}
		else				   	// Case when rotation is counter-clockwise: Faster speed, Full step			
		{
			if (coil_step == 7)
			{
				write_coils(coil_step7);
				coil_step = 5;
			}
			else if (coil_step == 5)
			{
				write_coils(coil_step5);
				coil_step = 3;
			}
			else if (coil_step == 3)
			{
				write_coils(coil_step3);
				coil_step = 1;
			}
			else
			{
				write_coils(coil_step1);
				coil_step = 7;
			}
			delay_ms(speed_delay);			// generates a delay
		}
	}
}	// End of main function code
